/* Dies ist die Beispielfunktion printCompoundString() aus folgendem Buch:
 *
 *  Autoren: Hans-Joachim Brede, Nicolai Josuttis,
 *           Sabine Lemberg, Achim Loerke
 *  Titel:   Programmieren mit OSF/Motif Version 2
 *  Verlag:  Addison-Wesley, 2. voellig ueberarbeitete Auflage 1995
 *  ISBN:    3-89319-727-3
 *
 * Copyright 1995 by Hans-Joachim Brede, Nicolai Josuttis, Sabine Lemberg and
 * Achim Loerke.  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software for personal
 * and educational use is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice
 * and this permission notice appear in supporting documentation, and that the
 * names of Addison-Wesley or the authors are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Addison-Wesley and the authors make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * ADDISON-WESLEY AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL ADDISON-WESLEY OR THE AUTHORS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Komponenten eines Compound-Strings als ASCII-Text ausgeben
 */
void printCompoundString (cstr)
XmString cstr;
{
    XmStringContext ck;         /* Kontext zum Auslesen */
    XmStringComponentType ct;   /* Komponenten-Typ */
    unsigned int len;           /* Werte-Groesse */
    XtPointer value;            /* Wert (oder NULL) */
    String s;                   /* Wert als String (oder NULL) */

    /* Kontext zum Auslesen initialisieren
     */
    if (! XmStringInitContext (&ck, cstr)) {
        fprintf (stderr, "Fehler in XmStringInitContext()\n");
        return;
    }

    do {
        /* naechste Komponente lesen */
        ct = XmStringGetNextTriple (ck, &len, &value);

        /* Laenge des Werts ausgeben */
        printf ("len: %2d ", len);

        /* Typ (und Wert) ausgeben */
        s = (String)value;
        switch (ct) {
          case XmSTRING_COMPONENT_UNKNOWN:
            printf (" - UNKNOWN\n");
            break;
          case XmSTRING_COMPONENT_CHARSET:
            printf (" - CHARSET/TAG (\"%s\")\n", s);
            break;
          case XmSTRING_COMPONENT_TEXT:
            printf (" - TEXT (\"%s\")\n", s);
            break;
          case XmSTRING_COMPONENT_DIRECTION:
            /* BEACHTE: Zeiger auf Wert wird zurueckgeliefert */
            {
              XmStringDirection d = *((XmStringDirection*)value);
              printf (" - DIRECTION (%s)\n",
                           d == XmSTRING_DIRECTION_L_TO_R ? "L_TO_R" :
                           d == XmSTRING_DIRECTION_R_TO_L ? "R_TO_L" :
                                                            "DEFAULT");
            }
            break;
          case XmSTRING_COMPONENT_SEPARATOR:
            printf (" - SEPARATOR\n");
            break;
          case XmSTRING_COMPONENT_TAB:
            printf (" - TAB\n");
            break;
          case XmSTRING_COMPONENT_LOCALE:
            printf (" - LOCALE (\"%s\")\n", s);
            break;
          case XmSTRING_COMPONENT_LOCALE_TEXT:
            /* MultiByte-Text ist normaler 8Bit-Text in Europa */
            printf (" - LOCALE_TEXT (\"%s\")\n", s);
            break;
          case XmSTRING_COMPONENT_WIDECHAR_TEXT:
            /* WideChar-Text als MultiByte-Text ausgeben */
            {
              char mbs[81];
              size_t len;
              len = wcstombs (mbs, (wchar_t*)value, 80);
              if (len == (size_t)-1) {
                  printf (" - WIDECHAR_TEXT (???)\n");
              } else{
                  mbs[len] = '\0';
                  printf (" - WIDECHAR_TEXT (\"%s\")\n", mbs);
              }
            }
            break;
          case XmSTRING_COMPONENT_RENDITION_BEGIN:
            printf (" - RENDITION_BEGIN (\"%s\")\n", s);
            break;
          case XmSTRING_COMPONENT_RENDITION_END:
            printf (" - RENDITION_END   (\"%s\")\n", s);
            break;
          case XmSTRING_COMPONENT_LAYOUT_PUSH:
            printf (" - LAYOUT_PUSH (\"%s\")\n", s);
            break;
          case XmSTRING_COMPONENT_LAYOUT_POP:
            printf (" - LAYOUT_POP  (\"%s\")\n", s);
            break;
        }

        /* Speicherplatz fuer den Wert freigeben */
        XtFree (s);
    } while (ct != XmSTRING_COMPONENT_END);
    printf (" - END\n");

    /* Kontext zum Auslesen freigeben
     */
    XmStringFreeContext (ck);
}

