/* Dies ist das Beispielprogramm uilbsp aus folgendem Buch:
 *
 *  Autoren: Hans-Joachim Brede, Nicolai Josuttis,
 *           Sabine Lemberg, Achim Loerke
 *  Titel:   Programmieren mit OSF/Motif Version 2
 *  Verlag:  Addison-Wesley, 2. voellig ueberarbeitete Auflage 1995
 *  ISBN:    3-89319-727-3
 *
 * Copyright 1995 by Hans-Joachim Brede, Nicolai Josuttis, Sabine Lemberg and
 * Achim Loerke.  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software for personal
 * and educational use is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice
 * and this permission notice appear in supporting documentation, and that the
 * names of Addison-Wesley or the authors are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Addison-Wesley and the authors make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * ADDISON-WESLEY AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL ADDISON-WESLEY OR THE AUTHORS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Allgemeine Header-Dateien einbinden
 */
#include <stdio.h>
#include <stdlib.h>

#include <Mrm/MrmPublic.h>

/* Klassenname der Applikation
 */
#define APPL_KLASSE    "XResbsp"

/* Vorwaerts-Deklarationen
 */
void initWidgetBaum();    /* Widgetbaum aufbauen */
void exitCB();            /* Exit-Callback */


main (argc, argv)
int   argc;
char* argv[];
{
    XtAppContext kontext;        /* Applikations-Umgebung */
    Widget       applShell;      /* Applikations-Shell */

    /* Initialisierungen und Applikations-Shell erzeugen
     */
    applShell = XtAppInitialize (&kontext, APPL_KLASSE,
                                 (XrmOptionDescRec*)NULL, 0,
                                 &argc, argv,
                                 (String*)NULL,
                                 (Arg*)NULL, 0);

    /* Widgetbaum unter der Applikations-Shell erzeugen
     */
    initWidgetBaum (applShell);

    /* Widgetbaum realisieren
     */
    XtRealizeWidget (applShell);

    /* Hauptschleife
     */
    XtAppMainLoop (kontext);
}


/* initWidgetBaum
 *
 * Widgetbaum unter der Applikations-Shell 
 * anhand UIL/UID-Beschreibung erzeugen
 */
void initWidgetBaum (elternWidget)
Widget elternWidget;          /* Eltern-Widget ist Applikations-Shell */
{
    MrmHierarchy hierarchie;       /* Mrm Hierarchie-ID */
    Widget       topWidget;        /* topWidget aus UIL */
    MrmType      topWidgetClass;   /* Klasse von topWidget */
    Cardinal     status;           /* Return-Status */

    static char *dateien [] = {    /* UID-Dateien */
        "uilbsp.uid"
    };
    static MrmRegisterArg callbacks[] = {    /* Callbacks */
        {"exitCB", (XtPointer)exitCB}
    };

    /* MRM initialisieren
     */
    MrmInitialize();

    /* MRM-Hierarchie mit UID-Dateien oeffnen
     */
    status = MrmOpenHierarchyPerDisplay (XtDisplay(elternWidget),
                                         XtNumber(dateien), dateien,
                                         (MrmOsOpenParamPtr*)NULL,
                                         &hierarchie);
    if (status != MrmSUCCESS) {
        fprintf (stderr, "UID-Dateien koennen nicht gelesen werden\n");
        exit (EXIT_FAILURE);
    }

    /* Callbacks fuer UIL registrieren
     */
    status = MrmRegisterNamesInHierarchy (hierarchie,
                                          callbacks, XtNumber(callbacks));
    if (status != MrmSUCCESS) {
        fprintf (stderr, "Callbacks koennen nicht registriert werden\n");
        exit (EXIT_FAILURE);
    }

    /* Widget-Hierarchie einlesen und aufbauen
     * Das oberste Widget dieser Hierarchie wird 'topWidget'
     * zugewiesen.
     */
    topWidget = NULL;
    status = MrmFetchWidget (hierarchie, "topWidget", elternWidget,
                             &topWidget, &topWidgetClass);
    if (status != MrmSUCCESS) {
        fprintf (stderr, "Widget-Hierarchie kann nicht erzeugt werden\n");
        exit (EXIT_FAILURE);
    }


    /* topWidget managen
     */
    XtManageChild (topWidget);
}


/* exitCB
 *
 * Exit-Callback: Programm mit fehlerfreiem Exit-Status beenden
 */
void exitCB (widget, clientDaten, aufrufDaten)
Widget    widget;         /* den Callback aufrufendes Widget */
XtPointer clientDaten;    /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;    /* Aufruf-Daten (hier nicht verwendet) */
{
    exit (EXIT_SUCCESS);
}

