/* Dies ist das Beispielprogramm uildemo aus folgendem Buch:
 *
 *  Autoren: Hans-Joachim Brede, Nicolai Josuttis,
 *           Sabine Lemberg, Achim Loerke
 *  Titel:   Programmieren mit OSF/Motif Version 2
 *  Verlag:  Addison-Wesley, 2. voellig ueberarbeitete Auflage 1995
 *  ISBN:    3-89319-727-3
 *
 * Copyright 1995 by Hans-Joachim Brede, Nicolai Josuttis, Sabine Lemberg and
 * Achim Loerke.  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software for personal
 * and educational use is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice
 * and this permission notice appear in supporting documentation, and that the
 * names of Addison-Wesley or the authors are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Addison-Wesley and the authors make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * ADDISON-WESLEY AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL ADDISON-WESLEY OR THE AUTHORS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Header-Datei
 */
/* Allgemeine C-Header-Dateien einbinden
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* Allgemeine X- und Motif-Header-Dateien einbinden
 */
#include <Xm/Xm.h>
#include <X11/Shell.h>
#include <Mrm/MrmPublic.h>      /* MRM-Funktionen und -Datentypen */

/* Header-Dateien fuer die verwendeten Widgetklassen einbinden
 */
#include <Xm/CascadeB.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/SelectioB.h>
#include <Xm/Separator.h>
#include <Xm/ToggleB.h>

/* Klassenname des Programms
 */
#define APPL_KLASSE    "UilDemo"

/* Konstante fuer App-Defaults-Version
 */
#define APP_DEFAULTS_VERSION    123

/* Variablen fuer die Groesse der Graphik. Da dies in xmdemo
 * Konstanten sind, werden sie hier der Uebersichtlichkeit halber
 * weiterhin gross geschrieben.
 */
unsigned int GRAPHIK_BREITE;
unsigned int GRAPHIK_HOEHE;


/* Globale Variablen
 */
XtAppContext kontext;              /* Applikations-Umgebung */
Display*     display;              /* Display */
MrmHierarchy hierarchie;           /* Hierarchie-ID */
Widget       graphikWidget;        /* Graphik-Widget */
Pixmap       graphikBitmap;        /* Bitmap fuer Graphik-Kopie */
String       graphikName = NULL;   /* aktueller Name der Graphik */
Widget       graphikNameWidget;    /* Widget fuer Graphik-Namen */
Widget       fsdWidget;            /* FileSelectionDialog zum Laden */

/* Attribute zum Malen
 */
Boolean      kontinuierlich;       /* kontinuierliches Malen ? */
int          malmodus;             /* Mal-Modus */
int VG_MODUS  = 0;                 /*  mit Vordergrund-Farbe */
int INV_MODUS = 1;                 /*  invertierend */
int HG_MODUS  = 2;                 /*  mit Hintergrund-Farbe */
#define MODUS_ANZ  3

/* GC's fuer Graphik-Fenster und Graphik-Bitmap
 */
GC fensterGC[MODUS_ANZ];     /* Fenster GC's */
GC bitmapGC[MODUS_ANZ];      /* Bitmap GC's */
GC copyGC;                   /* GC zum Kopieren von Bitmap ins Fenster */

/* Fallback-Ressourcen (nur das Notwendigste)
 */
static String fallbackRessourcen[] = {

   "*dateiname.labelString:       Dateiname: ",
   "*graphikName.alignment:       alignment_beginning",

   "*scrolledGraphik.scrollingPolicy:    automatic",
   "*scrolledGraphik.scrollBarPlacement: bottom_right",

   "*form.horizontalSpacing:             4",
   "*form.verticalSpacing:               4",
   "*graphikName.leftOffset:             0",
   "*form.backgroundPixmap:              25_foreground",
   "*scrolledGraphik.backgroundPixmap:   25_foreground",
   "*scrolledGraphik.XmDrawingArea.backgroundPixmap: 25_foreground",

   "*vordergrund.labelString:     Schwarz",
   "*invertieren.labelString:     Invers",
   "*hintergrund.labelString:     Weiss",
   "*breitenMenue.labelString:    Breite:",
   "*graphik.background:          white",
   "*graphik.foreground:          black",

   "*okLabelString:               OK",
   "*cancelLabelString:           Abbruch",
   "*hilfeDialog.messageString:   Keine Hilfe noetig",
   "*ladenDialog.dirMask:         *.xbm",
   "*speichernAlsDialog.selectionLabelString: Dateiname ?",

   NULL
};

/* Vorwaerts-Deklarationen
 */
void falscheParameter();
void initWidgetBaum();
void liesApplikationsRessourcen();
void setzeGraphikName();
void malmodusCB();
void linienbreiteCB();
void kontinuierlichCB();
void initGCs();
void inputCB();
void motionEventHandler();
void exposeCB();
void loescheGraphik();
void popupMenueAusgeben();
void invertierenCB();
void loeschenCB();
void hilfeCB();
void ladenCB();
void fsdFertigCB();
Boolean ladeGraphik();
void unmanageCB();
void speichernCB();
Boolean speicherGraphik();
void speichernAlsCB();
void eingabeCB();
void beendenCB();


/* Module
 */

/* Optionen
 */
static XrmOptionDescRec optionen[] = {

    /* Dateiname der zu editierenden Graphik-Bitmap
     */
    { "-datei", ".datei", XrmoptionSepArg, NULL },

#if XmVersion <= 1002
    /* '-font' und '-fn' setzen bis Motif 1.2 Font-Listen
     */
    { "-font", "*fontList", XrmoptionSepArg, NULL },
    { "-fn",   "*fontList", XrmoptionSepArg, NULL }
#endif
};


main (argc, argv)
int   argc;
char* argv[];
{
    Widget   applShell;      /* Applikations-Shell */

    /* Initialisierungen und Applikations-Shell erzeugen
     */
    applShell = XtAppInitialize (&kontext, APPL_KLASSE,
                                 optionen, XtNumber(optionen),
                                 &argc, argv,
                                 fallbackRessourcen,
                                 (Arg*)NULL, 0);

    /* Warnung fuer nicht verarbeitete Parameter ausgeben
     */
    if (argc != 1) {
        falscheParameter (argc-1, argv+1);
    }

    /* !!! Motif Resource Manager initialisieren
     */
    MrmInitialize();

    /* Display-Variable initialisieren
     */
    display = XtDisplay (applShell);

    /* Widgetbaum unter der Applikations-Shell erzeugen
     */
    initWidgetBaum (applShell);

    /* Applikations-Ressourcen auslesen und verarbeiten
     */
    liesApplikationsRessourcen (applShell);

    /* Widgetbaum realisieren
     */
    XtRealizeWidget (applShell);

    /* Graphik leeren
     */
    loescheGraphik();

    /* Falls vorhanden und lesbar, erste Graphik laden
     */
    if (access(graphikName,R_OK) == 0) {
        ladeGraphik (graphikName);
    }

    /* Hauptschleife
     */
    XtAppMainLoop (kontext);
}


void falscheParameter (argcFalsch, argvFalsch)
int   argcFalsch;       /* Anzahl falscher Parameter */
char* argvFalsch[];     /* Array  falscher Parameter */
{
    int    zeichenAnzahl;
    String warnung;
    int    i;

    /* Anzahl der Zeichen fuer den Warnungs-Text ermitteln und
     * entsprechenden Speicherplatz anfordern
     */
    zeichenAnzahl = 0;
    for (i=0; i<argcFalsch; i++) {
        zeichenAnzahl += strlen(argvFalsch[i]) + 1;
    }
    warnung = XtMalloc ((zeichenAnzahl+30) * sizeof(char));

    /* Warnungs-Text "Falsche Parameter: ..." eintragen */
    strcpy (warnung, "Falsche Parameter:");
    for (i=0; i<argcFalsch; i++) {
        strcat (warnung, " ");
        strcat (warnung, argvFalsch[i]);
    }

    /* Warnung ausgeben
     */
    XtAppWarning (kontext, warnung);
    
    /* Angeforderten Speicherplatz wieder freigeben
     */
    XtFree (warnung);
}

static void setzeWerteAusUIL()
{
    XtPointer breiteZeiger, hoeheZeiger;
    MrmCode typ;

    if ((MrmFetchLiteral(hierarchie,"GRAPHIK_BREITE",display,
                         &breiteZeiger,&typ) != MrmSUCCESS) ||
        (typ != MrmRtypeInteger)) {
        XtAppError(kontext,
                   "Literal GRAPHIK_BREITE kann nicht gelesen werden");
    }
    GRAPHIK_BREITE = *(int*)breiteZeiger;
    
    if ((MrmFetchLiteral(hierarchie,"GRAPHIK_HOEHE",display,
                         &hoeheZeiger,&typ) != MrmSUCCESS)  ||
        (typ != MrmRtypeInteger)) {
        XtAppError(kontext,
                   "Literal GRAPHIK_HOEHE kann nicht gelesen werden");
    }
    GRAPHIK_HOEHE = *(int*)hoeheZeiger;
}

static void graphikNameCB (widget, clientDaten, aufrufDaten)
Widget    widget;
XtPointer clientDaten,      /* hier nicht verwendet */
          aufrufDaten;      /* und hier auch nicht  */
{
    graphikNameWidget = widget;
}

static void setzeKontinuierlichCB (widget, clientDaten, aufrufDaten)
Widget    widget;
XtPointer clientDaten,      /* hier nicht verwendet */
          aufrufDaten;      /* und hier auch nicht  */
{
    Arg      args[1];
    Cardinal n;
    /* Default-Zustand (an oder aus) kann von aussen als Ressource
     * gesetzt werden.
     * Dieser muss ausgelesen werden, damit die davon betroffene
     * Variable 'kontinuierlich' entsprechend gesetzt wird.
     */
    n = 0;
    XtSetArg (args[n], XmNset, &kontinuierlich);  n++;
    XtGetValues (widget, args, n);    
}

static void graphikPopupCB (widget, clientDaten, aufrufDaten)
Widget    widget;
XtPointer clientDaten,      /* hier nicht verwendet */
          aufrufDaten;      /* und hier auch nicht  */
{
    /* Event-Handler zum Ausgeben des Popup-Menues installieren.
     * Das auszugebende Popup-Menue wird als Client-Datum uebergeben.
     */
    XtAddEventHandler (XtParent(XtParent(widget)), ButtonPressMask, False,
                       popupMenueAusgeben, (XtPointer)widget);
}

static void hilfeDialogCB (widget, clientDaten, aufrufDaten)
Widget    widget;
XtPointer clientDaten,      /* hier nicht verwendet */
          aufrufDaten;      /* und hier auch nicht  */
{
    /* Cancel- und Help-Button werden nicht gebraucht
     */

    XtUnmanageChild (XmMessageBoxGetChild(widget,
                                          XmDIALOG_HELP_BUTTON));
    XtUnmanageChild (XmMessageBoxGetChild(widget,
                                          XmDIALOG_CANCEL_BUTTON));
}

static void fsdInitCB (widget, clientDaten, aufrufDaten)
Widget    widget;
XtPointer clientDaten,      /* hier nicht verwendet */
          aufrufDaten;      /* und hier auch nicht  */
{
    fsdWidget = widget;

    /* Der Help-Button wird nicht verwendet
     */
    XtUnmanageChild (XmFileSelectionBoxGetChild (fsdWidget,
                                                 XmDIALOG_HELP_BUTTON));
}

static void speichernAlsDialogCB (widget, clientDaten, aufrufDaten)
Widget    widget;
XtPointer clientDaten,      /* hier nicht verwendet */
          aufrufDaten;      /* und hier auch nicht  */
{
    /* Help-Button wird nicht gebraucht
     */
    XtUnmanageChild (XmSelectionBoxGetChild(widget,
                                            XmDIALOG_HELP_BUTTON));
}

static void initGraphikCB (widget, clientDaten, aufrufDaten)
Widget    widget;
XtPointer clientDaten,      /* hier nicht verwendet */
          aufrufDaten;      /* und hier auch nicht  */
{
    static XPoint alteKoords;      /* jeweils vorherige Koordinaten */
    graphikWidget = widget;

    /* Event-Handler fuer Motion-Events installieren
     */
    XtAddEventHandler (graphikWidget, Button1MotionMask, False,
                       motionEventHandler, (XtPointer)&alteKoords);

    /* Input-Callback installieren
     */
    XtAddCallback (graphikWidget, XmNinputCallback,
                   inputCB, (XtPointer)&alteKoords);

    /* Graphik-Bitmap fuer Kopie des Fensterinhaltes erzeugen
     */
    graphikBitmap = XCreatePixmap (display,
                            RootWindowOfScreen(XtScreen(graphikWidget)),
                            GRAPHIK_BREITE, GRAPHIK_HOEHE, 1);

    /* GCs zum Malen in Graphik-Fenster und Graphik-Bitmap erzeugen
     */
    initGCs (graphikWidget);
}

/* Anbindungen zwischen UIL und C */
static String uidDateien[] = {        /* Namen der UID-Dateien */
    /* der Rahmen */
    "main.uid",
    /* die Menu-Komponente */
    "menu.uid",
    "filesel.uid",
    "speichernAls.uid",
    "hilfe.uid",
    /* das Arbeitsfeld */
    "form.uid",
    "attribute.uid",
    "malmodus.uid",
    "breiten.uid",
    "graphik.uid",
};

static MrmRegisterArg namenWerteListe[] = { /* Zuordnung UIL-C-Namen */
    /* aus menu.uil */
    {"ladenCB",                (XtPointer)ladenCB},
    {"speichernCB",            (XtPointer)speichernCB},
    {"speichernAlsCB",         (XtPointer)speichernAlsCB},
    {"beendenCB",              (XtPointer)beendenCB},
    {"hilfeCB",                (XtPointer)hilfeCB},

    /* aus form.uil */
    {"graphikNameCB",          (XtPointer)graphikNameCB},
    {"initGraphikCB",          (XtPointer)initGraphikCB},
    {"kontinuierlichCB",       (XtPointer)kontinuierlichCB},
    {"setzeKontinuierlichCB",  (XtPointer)setzeKontinuierlichCB},

    /* aus malmodus.uil */
    {"malmodusCB",             (XtPointer)malmodusCB},


    /* und Identifier fuer malmodus.uil */
    {"VG_MODUS",               (XtPointer)&VG_MODUS},
    {"INV_MODUS",              (XtPointer)&INV_MODUS},
    {"HG_MODUS",               (XtPointer)&HG_MODUS},

    /* aus breiten.uil */
    {"linienbreiteCB",         (XtPointer)linienbreiteCB},

    /* aus graphik.uil */
    {"exposeCB",               (XtPointer)exposeCB},
    {"invertierenCB",          (XtPointer)invertierenCB},
    {"loeschenCB",             (XtPointer)loeschenCB},
    {"graphikPopupCB",         (XtPointer)graphikPopupCB},

    /* aus filesel.uil */
    {"fsdInitCB",              (XtPointer)fsdInitCB},
    {"fsdFertigCB",            (XtPointer)fsdFertigCB},
    {"unmanageCB",             (XtPointer)unmanageCB},

    /* aus hilfe.uil */
    {"hilfeDialogCB",          (XtPointer)hilfeDialogCB},

    /* aus speichernAls.uil */
    {"speichernAlsDialogCB",   (XtPointer)speichernAlsDialogCB},
    {"eingabeCB",              (XtPointer)eingabeCB},
};

void initWidgetBaum (applShell)
Widget applShell;              /* Applikations-Shell */
{
    Widget        mainW = (Widget)NULL;   /* temporaere Widgets */
    MrmType       widgetType;             /* Type eines Widgets */


    if ( MrmOpenHierarchyPerDisplay(XtDisplay(applShell),
                                    XtNumber(uidDateien),uidDateien,
                                    (MrmOsOpenParamPtr*)NULL,
                                    &hierarchie) != MrmSUCCESS ) {
        XtAppError(kontext,
                   "MRM-Hierarchie konnte nicht geoeffnet werden");
    }

    if ( MrmRegisterNamesInHierarchy(hierarchie,
                    namenWerteListe,
                    XtNumber(namenWerteListe)) != MrmSUCCESS ) {
        XtAppError(kontext,"MRM: Namen konnten nicht registriert werden");
    }

    /* einige Werte muessen vorher aus den UIL-Modulen geholt werden */
    setzeWerteAusUIL();

    /* XmMainWindow-Widget lesen, dabei werden alle Kinder mit aufgebaut
     */
    if ( MrmFetchWidget(hierarchie,"main",applShell,
                        &mainW,&widgetType) != MrmSUCCESS ) {
        XtAppError(kontext,"main konnte nicht geladen werden");
    }
    XtManageChild(mainW);
}

/* Konstanten fuer Applikations-Ressource definieren
 */
#define XbspNdatei               "datei"
#define XbspCDatei               "Datei"
#define XbspNappDefaultsVersion  "appDefaultsVersion"
#define XbspCAppDefaultsVersion  "AppDefaultsVersion"

/* Struktur-Datentyp fuer alle
 * Applikations-Ressourcen definieren
 */
typedef struct {
    String datei;      /* Appl.-Res. 'datei' */
    int    version;    /* Appl.-Res. 'appDefaultsVersion' */
} ApplResTyp;

/* Ressource-Liste mit allen Eigenschaften der
 * Applikations-Ressourcen definieren
 */
static XtResource ressourcen[] = {
  /* Name, Klasse, Typ, Size, Offset, Default-Typ, Default-Wert */

    /* Appl.-Ressource 'datei' (Default-Wert: "uildemo.xbm")
     */
    { XbspNdatei, XbspCDatei,
      XmRString, sizeof(String),
      XtOffset(ApplResTyp*,datei),
      XmRString, "uildemo.xbm" },

    /* Appl.-Ressource 'appDefaultsVersion' (Default-Wert: 0)
     */
    { XbspNappDefaultsVersion, XbspCAppDefaultsVersion,
      XmRInt, sizeof(int),
      XtOffset(ApplResTyp*,version),
      XmRString, "0" }
};

void liesApplikationsRessourcen (applShell)
Widget applShell;
{
   /* Variable fuer Applikations-Ressourcen
    */
   ApplResTyp applRessourcen;
   /* Applikations-Ressourcen auslesen
    */
   XtGetApplicationResources (applShell, &applRessourcen,
                              ressourcen, XtNumber(ressourcen),
                              (Arg*)NULL, 0);

   /* Applikations-Ressource 'datei' verarbeiten
    * (Name der ersten Bitmap-Datei)
    */
   setzeGraphikName (applRessourcen.datei);

   /* Applikations-Ressource 'appDefaultsVersion' verarbeiten
    * (Version der Application-Defaults)
    */
   if (applRessourcen.version == 0) {
       /* Default-Wert, also keine Appl.-Defaults installiert
        */
       XtAppWarning (kontext, "Keine Application-Defaults installiert");
   }
   else if (applRessourcen.version != APP_DEFAULTS_VERSION) {
       /* Falscher Wert, also falsche Appl.-Defaults installiert
        */
       XtAppWarning (kontext, "Falsche Application-Defaults installiert");
   }
}

void setzeGraphikName (name)
String name;       /* neuer Name der Graphik */
{
    XmString label;        /* Compound-String fuer das Label */
    Arg      args[10];     /* Argumente-Liste */
    Cardinal n;            /* Lauf-Index */

    /* Alten Graphik-Namen freigeben und
     * neuen Namen setzen (falls nicht identisch)
     */
    if (graphikName != name) {
        XtFree (graphikName);
        graphikName = XtNewString (name);
    }

    /* Neues Label fuer Graphik-Name-Widget erzeugen, setzen
     * und freigeben */
    label = XmStringCreate (graphikName, XmSTRING_DEFAULT_CHARSET);
    n = 0;
    XtSetArg (args[n], XmNlabelString, label);  n++;
    XtSetValues (graphikNameWidget, args, n);
    XmStringFree (label);
}

void malmodusCB (widget, clientDaten, aufrufDaten)
Widget    widget;         /* den Callback aufrufendes Widget */
XtPointer clientDaten;    /* Client-Daten (hier Mal-Modus) */
XtPointer aufrufDaten;    /* Aufruf-Daten mit Callback-Informationen */
{
    /* Falls XmToggleButton gesetzt wurde, dessen Mal-Modus
     * eintragen (wird als Client-Datum uebergeben)
     */
    if (((XmToggleButtonCallbackStruct*)aufrufDaten)->set) {
        /* !!! UIL liefert immer Zeiger */
        malmodus = *((int*)clientDaten);  
    }
}

void linienbreiteCB (widget, clientDaten, aufrufDaten)
Widget    widget;           /* den Calllback aufrufendes Widget */
XtPointer clientDaten,      /* Client-Daten (hier Linienbreite) */
          aufrufDaten;      /* Aufruf-Daten (hier nicht verwendet) */
{
    XGCValues werte;      /* Struktur zum Setzen von GC-Attributen */
    int       i;          /* Laufvariable */

    /* Client-Datum enthaelt Linienbreite
     */
    /* !!! UIL liefert immer Zeiger !!! */
    werte.line_width = *((int*)clientDaten); 
    
    /* Fuer jeden Mal-Modus die GC's anpassen
     */
    for (i=0; i<MODUS_ANZ; i++) {
        XChangeGC (display, fensterGC[i], GCLineWidth, &werte);
        XChangeGC (display, bitmapGC[i],  GCLineWidth, &werte);
    }
}

void kontinuierlichCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten mit Callback-Informationen */
{
    /* Davon betroffene Variable 'kontinuierlich' entsprechend
     * neuem Zustand des Buttons (Komponente 'set' in den
     * Callback-Informationen) setzen.
     */
    kontinuierlich = ((XmToggleButtonCallbackStruct*)aufrufDaten)->set;
}

void initGCs (widget)
Widget widget;        /* Widget, fuer das die GC's gebraucht werden */
{
    Pixel         fg, bg;     /* Farbwerte fuer Vorder- und Hintergrund */
    XGCValues     werte;      /* Struktur zum Setzen von GC-Attributen */
    unsigned long maske;      /* Maske fuer GC-Attribute */
    Window        root;       /* Root-Window */
    Arg           args[10];   /* Argumente-Liste */
    Cardinal      n;          /* Lauf-Index */

    /* Farbwerte des Graphik-Widgets auslesen
     * (es geht auch rot auf gruen)
     */
    n = 0;
    XtSetArg (args[n], XmNforeground, &fg);  n++;
    XtSetArg (args[n], XmNbackground, &bg);  n++;
    XtGetValues (widget, args, n);

    /* GC's fuer das Graphik-Fenster und die Graphik-Bitmap erzeugen.
     * - Fuer jeden Mal-Modus einen GC.
     * - Da das Fenster zum Widget auch mehrere Farbebenen haben kann,
     *   muessen die GC's von Fenster und Bitmap verschieden sein.
     * - Da in den GC's jeweils die Linienbreite geaendert wird,
     *   muessen es aenderbare (keine Shared-GC's) sein.
     */
    
    /* Da das Graphik-Widget noch nicht realisiert ist, wird das
     * Root-Window beim Erzeugen der GC's als Drawable, das
     * Bildschirm und Anzahl der Farbebenen des Graphik-Fensters
     * identifiziert, verwendet.
     */
    root = RootWindowOfScreen(XtScreen(widget));

    /* Runde Linienenden
     */
    werte.cap_style = CapRound;

    /* GC's fuer das Malen mit Vordergrund-Farbwert erzeugen
     */
    werte.foreground = fg;
    maske = GCForeground|GCCapStyle;
    fensterGC[VG_MODUS] = XCreateGC (display, root, maske, &werte);
    werte.foreground = 1;
    bitmapGC[VG_MODUS]  = XCreateGC (display, graphikBitmap,
                                     maske, &werte);

    /* GC's fuer invertierendes Malen erzeugen
     */
    werte.foreground = fg^bg;
    werte.function = GXxor;
    maske = GCForeground|GCFunction|GCCapStyle;
    fensterGC[INV_MODUS] = XCreateGC (display, root, maske, &werte);
    werte.foreground = 1^0;
    bitmapGC[INV_MODUS]  = XCreateGC (display, graphikBitmap,
                                      maske, &werte);

    /* GC's fuer das Malen mit Hintergrund-Farbwert erzeugen
     */
    werte.foreground = bg;
    maske = GCForeground|GCCapStyle;
    fensterGC[HG_MODUS] = XCreateGC (display, root, maske, &werte);
    werte.foreground = 0;
    bitmapGC[HG_MODUS] =  XCreateGC (display, graphikBitmap,
                                     maske, &werte);

    /* GC zum Kopieren der Bitmap-Daten ins Graphik-Fenster erzeugen
     */
    werte.foreground = fg;
    werte.background = bg;
    maske = GCForeground|GCBackground;
    copyGC = XCreateGC (display, root, maske, &werte);
}

void inputCB (widget, clientDaten, aufrufDaten)
Widget    widget;           /* den Callback aufrufendes Widget */
XtPointer clientDaten,      /* Zeiger auf alte Koordinaten */
          aufrufDaten;      /* Callback-Informationen */
{
    XmDrawingAreaCallbackStruct* DAinfos;
    int                          x, y;

    /* Callback-Informationen mit Event-Struktur
     */
    DAinfos = (XmDrawingAreaCallbackStruct*)aufrufDaten;

    /* Reaktion nur auf Druecken von Mausbutton 1
     */
    if (DAinfos->event->type == ButtonPress &&
        DAinfos->event->xbutton.button == Button1) {
          /* Koordinaten auslesen
           */
          x = DAinfos->event->xbutton.x;
          y = DAinfos->event->xbutton.y;

          /* Ersten Punkt malen
           * (muss als Linie gemalt werden, da sonst Breite 1)
           */
          XDrawLine (display, XtWindow(widget), fensterGC[malmodus],
                     x, y, x, y);
          XDrawLine (display, graphikBitmap, bitmapGC[malmodus],
                     x, y, x, y);

          /* Start-Koordinaten als neue "vorherige Koordinaten"
           * merken
           */
          ((XPoint*)clientDaten)->x = x;
          ((XPoint*)clientDaten)->y = y;
    }
}

void motionEventHandler (widget, clientDaten, eventAdr)
Widget    widget;         /* den Event-Handler aufrufendes Widget */
XtPointer clientDaten;    /* Zeiger auf vorherige Koordinaten */
XEvent*   eventAdr;       /* Zeiger auf ausloesenden X-Event */
{
    int x_von, y_von, x_nach, y_nach;

    /* Maus-Koordinaten auslesen
     */
    x_nach = eventAdr->xmotion.x;
    y_nach = eventAdr->xmotion.y;

    if (!kontinuierlich) {
        /* Nur den Punkt aus dem Motion-Event malen
         */
        x_von = x_nach;
        y_von = y_nach;
    }
    else {
        /* Vorherige Koordinaten auslesen
         */
        x_von = ((XPoint*)clientDaten)->x;
        y_von = ((XPoint*)clientDaten)->y;

        if (malmodus == INV_MODUS) {
            /* Da erster Punkt bereits invertiert ist und
             * nochmal invertiert wird, hebt sich dies auf.
             * Deswegen wird er hier nochmal invertiert.
             */
            XDrawLine (display, XtWindow(widget), fensterGC[malmodus],
                       x_von, y_von, x_von, y_von);
            XDrawLine (display, graphikBitmap, bitmapGC[malmodus],
                       x_von, y_von, x_von, y_von);
        }
    }
    /* Neue Linie bzw. neuen Punkt malen
     */
    XDrawLine (display, XtWindow(widget), fensterGC[malmodus],
               x_von, y_von, x_nach, y_nach);

    XDrawLine (display, graphikBitmap, bitmapGC[malmodus],
               x_von, y_von, x_nach, y_nach);
    /* Neue Koordinaten als 'vorherige Koordinaten' merken
     */
    ((XPoint*)clientDaten)->x = x_nach;
    ((XPoint*)clientDaten)->y = y_nach;
}

void exposeCB (widget, clientDaten, aufrufDaten)
Widget  widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;   /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;   /* Aufruf-Daten mit Callback-Informationen */
{
    XmDrawingAreaCallbackStruct* DAinfos;

    /* Callback-Informationen mit Event-Struktur
     */
    DAinfos = (XmDrawingAreaCallbackStruct*)aufrufDaten;

    /* Im Expose-Event angegebenen Fenster-Ausschnitt 
     * von Graphik-Bitmap ins Graphik-Fenster kopieren
     */
    XCopyPlane (display, graphikBitmap, XtWindow(widget), copyGC,
                DAinfos->event->xexpose.x, DAinfos->event->xexpose.y,
                DAinfos->event->xexpose.width,
                DAinfos->event->xexpose.height,
                DAinfos->event->xexpose.x, DAinfos->event->xexpose.y,
                1);
}

void loescheGraphik()
{
   /* Graphik-Fenster und Graphik-Bitmap durch Bemalen mit
    * Hintergrundfarbe loeschen
    */
   XFillRectangle (display, XtWindow(graphikWidget), fensterGC[HG_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
   XFillRectangle (display, graphikBitmap, bitmapGC[HG_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
}

void popupMenueAusgeben (widget, clientDaten, eventAdr)
Widget    widget;          /* den Event-Handler aufrufendes Widget */
XtPointer clientDaten;     /* auszugebendes Popup-Menue */
XEvent*   eventAdr;        /* Zeiger auf ausloesenden X-Event */
{
    /* Nur Mausbutton 3 gibt Popup-Menue aus
     */
    if (eventAdr->xbutton.button != Button3) {
        return;
    }

    /* Popup-Menue anhand Mausposition positionieren
     */
    XmMenuPosition ((Widget)clientDaten, &eventAdr->xbutton);

    /* Popup-Menue ausgeben
     */
    XtManageChild ((Widget)clientDaten);
}

void invertierenCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
   /* Graphik-Fenster und Graphik-Bitmap durch Bemalen mit
    * invertierendem GC invertieren
    */
   XFillRectangle (display, XtWindow(graphikWidget), fensterGC[INV_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
   XFillRectangle (display, graphikBitmap, bitmapGC[INV_MODUS],
                   0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE);
}

void loeschenCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Graphik-Fenster und Graphik-Bitmap loeschen
     */
    loescheGraphik();
}

void hilfeCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    static Widget hilfeWidget = NULL;
    MrmType widgetType;

    /* Falls noch nicht geschehen:
     *  XmMessageBox erzeugen
     */

    if ((hilfeWidget==NULL) && 
        (MrmFetchWidget(hierarchie,"hilfeDialog",widget,
                        &hilfeWidget,&widgetType) != MrmSUCCESS) ) {
        XtAppError(kontext,"hilfeDialog kann nicht geladen werden");
    }

    /* XmMessageBox ausgeben
     */
    XtManageChild (hilfeWidget);
}

void ladenCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Angezeigte Dateien auf aktuellen Stand bringen
     */
    XmFileSelectionDoSearch (fsdWidget, (XmString)NULL);
    /* FileSelectionDialog ausgeben
     */
    XtManageChild (fsdWidget);
}

void fsdFertigCB (widget, clientDaten, aufrufDaten)
Widget  widget;            /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten mit Callback-Informationen */
{
    XmFileSelectionBoxCallbackStruct* FSBinfos;
    String                            dateiname;

    /* Aufruf-Daten der FileSelectionBox enthalten unter anderem
     * in der Komponente 'value' den eingegebenen Dateinamen
     */
    FSBinfos = (XmFileSelectionBoxCallbackStruct*)aufrufDaten;

    /* Dateinamen auslesen (Kopie mit eigenem Speicherplatz)
     */
    XmStringGetLtoR (FSBinfos->value, XmSTRING_DEFAULT_CHARSET,
                     &dateiname);

    /* Graphik dazu laden
     */
    ladeGraphik (dateiname);

    /* Speicherplatz fuer ausgelesenen Dateinamen wieder freigeben
     */
    XtFree (dateiname);
}

Boolean ladeGraphik (dateiname)
String dateiname;    /* Dateiname fuer zu ladende Graphik */
{
    Pixmap       bitmap;          /* geladene Bitmap */
    int          status;          /* Laden geklappt ? */
    unsigned int breite, hoehe;   /* Groesse der geladenen Bitmap */
    int          dummyInt;        /* Dummy-Parameter */

    /* Bitmap-Datei aus 'dateiname' einlesen
     */
    status = XReadBitmapFile (display, graphikBitmap, dateiname,
                              &breite, &hoehe, &bitmap,
                              &dummyInt, &dummyInt);

    /* Hat das Laden geklappt ?
     */
    if (status != BitmapSuccess) {
        XBell (display, 0);
        XtAppWarning (kontext, "Bitmap-Datei nicht lesbar");
        return False;
    }

    /* Stimmt die Groesse ?
     */
    if (breite != GRAPHIK_BREITE || hoehe != GRAPHIK_HOEHE) {
        XBell (display, 0);
        XtAppWarning (kontext, "Bitmap-Datei hat falsche Groesse");
        return False;
    }

    /* Alte Bitmap freigeben und neue eintragen
     */
    XFreePixmap (display, graphikBitmap);
    graphikBitmap = bitmap;

    /* Inhalt ins Graphik-Fenster kopieren
     */
    XCopyPlane (display, graphikBitmap, XtWindow(graphikWidget), copyGC,
                0, 0, GRAPHIK_BREITE, GRAPHIK_HOEHE, 0, 0, 1);

    /* Dateinamen der geladenen Graphik setzen
     */
    setzeGraphikName (dateiname);

    return True;
}

void unmanageCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Widget, das Callback aufrief, unmanagen
     */
    XtUnmanageChild (widget);
}

void speichernCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Graphik abspeichern
     */
    speicherGraphik (graphikName);
}

Boolean speicherGraphik (dateiname)
String dateiname;     /* Dateiname der abzuspeichernden Graphik */
{
    int status;          /* Speichern geklappt ? */

    /* Bitmap-Daten in 'dateiname' abspeichern
     */

    status = XWriteBitmapFile (display, dateiname, graphikBitmap,
                               GRAPHIK_BREITE, GRAPHIK_HOEHE, -1, -1);

    /* Hat das Speichern geklappt ?
     */
    if (status != BitmapSuccess) {
        XBell (display, 0);
        XtAppWarning (kontext, "Bitmap-Daten nicht abspeicherbar");
        return False;
    }

    return True;
}

void speichernAlsCB (widget, clientDaten, aufrufDaten)
Widget    widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;     /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;     /* Aufruf-Daten (hier nicht verwendet) */
{
    static Widget speichernAlsWidget = NULL;
    MrmType widgetType;

    /* Falls noch nicht geschehen:
     *  XmPromptDialog zum Eingeben eines Dateinamens erzeugen
     *  (Rest des Programms wird bei Ausgabe gesperrt)
     */
    if ((speichernAlsWidget==NULL) && 
        (MrmFetchWidget(hierarchie,
                        "speichernAlsDialog",widget,
                        &speichernAlsWidget,
                        &widgetType) != MrmSUCCESS) ) {
        XtAppError(kontext,
                   "speichernAlsDialog kann nicht geladen werden");
    }
    /* XmPromptDialog ausgeben
     */
    XtManageChild (speichernAlsWidget);
}

void eingabeCB (widget, clientDaten, aufrufDaten)
Widget  widget;          /* den Callback aufrufendes Widget */
XtPointer clientDaten;   /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;   /* Aufruf-Daten mit Callback-Informationen */
{
    XmSelectionBoxCallbackStruct* SBinfos;
    String                        dateiname;

    /* Aufruf-Daten der SelectionBox enthalten unter anderem
     * den eingegebenen Dateinamen
     */
    SBinfos = (XmSelectionBoxCallbackStruct*)aufrufDaten;

    /* Dateinamen auslesen (Kopie mit eigenem Speicherplatz)
     */
    XmStringGetLtoR (SBinfos->value, XmSTRING_DEFAULT_CHARSET,
                     &dateiname);

    /* Graphik dort abspeichern
     */
    speicherGraphik (dateiname);

    /* Speicherplatz fuer ausgelesenen Dateinamen wieder freigeben
     */
    XtFree (dateiname);
}

void beendenCB (widget, clientDaten, aufrufDaten)
Widget  widget;        /* den Callback aufrufendes Widget */
XtPointer clientDaten;   /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;   /* Aufruf-Daten (hier nicht verwendet) */
{
    /* Programm mit fehlerfreiem Status beenden
     */
    exit (EXIT_SUCCESS);
}

