/* Dies ist das Beispielprogramm xresbsp aus folgendem Buch:
 *
 *  Autoren: Hans-Joachim Brede, Nicolai Josuttis,
 *           Sabine Lemberg, Achim Loerke
 *  Titel:   Programmieren mit OSF/Motif Version 2
 *  Verlag:  Addison-Wesley, 2. voellig ueberarbeitete Auflage 1995
 *  ISBN:    3-89319-727-3
 *
 * Copyright 1995 by Hans-Joachim Brede, Nicolai Josuttis, Sabine Lemberg and
 * Achim Loerke.  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software for personal
 * and educational use is hereby granted without fee, provided that the above
 * copyright notice appear in all copies and that both that copyright notice
 * and this permission notice appear in supporting documentation, and that the
 * names of Addison-Wesley or the authors are not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Addison-Wesley and the authors make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * ADDISON-WESLEY AND THE AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 * IN NO EVENT SHALL ADDISON-WESLEY OR THE AUTHORS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

/* Allgemeine Header-Dateien einbinden
 */
#include <stdio.h>
#include <stdlib.h>
#include <Xm/Xm.h>

/* Header-Dateien fuer die verwendeten Widgetklassen einbinden
 */
#include <Xm/RowColumn.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>

/* Klassenname der Applikation
 */
#define APPL_KLASSE    "XResbsp"

/* Vorwaerts-Deklarationen
 */
void initWidgetBaum();    /* Widgetbaum aufbauen */
void inversCB();          /* Invers-Callback */
void exitCB();            /* Exit-Callback */


main (argc, argv)
int   argc;
char* argv[];
{
    XtAppContext kontext;        /* Applikations-Umgebung */
    Widget       applShell;      /* Applikations-Shell */

    /* Initialisierungen und Applikations-Shell erzeugen
     */
    applShell = XtAppInitialize (&kontext, APPL_KLASSE,
                                 (XrmOptionDescRec*)NULL, 0,
                                 &argc, argv, (String*)NULL,
                                 (Arg*)NULL, 0);

    /* Widgetbaum unter der Applikations-Shell erzeugen
     */
    initWidgetBaum (applShell);

    /* Widgetbaum realisieren
     */
    XtRealizeWidget (applShell);

    /* Hauptschleife
     */
    XtAppMainLoop (kontext);
}


/* initWidgetBaum
 *  - Widgetbaum unter der Applikations-Shell erzeugen
 */
void initWidgetBaum (applShell)
Widget applShell;          /* Applikations-Shell */
{
    Widget   box, exitButton;    /* temporaere Widgets */
    Arg      args[10];           /* Argumente-Liste */
    Cardinal n;                  /* Lauf-Index */
    static XtCallbackRec invers_cbliste[] = {
            {inversCB, NULL},
            {NULL, NULL}
    };                           /* Invers-Callback-Liste */

    /* XmRowColumn-ContainerWidget erzeugen
     */
    box = XtCreateManagedWidget ("box", xmRowColumnWidgetClass,
                                 applShell, (Arg*)NULL, 0);


    /* XmLabel-Widget als Titelzeile erzeugen
     */
    XtCreateManagedWidget ("titel", xmLabelWidgetClass,
                           box, (Arg*)NULL, 0);


    /* XmPushButton-Widget als sich selbst invertierender
     * Button erzeugen
     */
    n = 0;
    XtSetArg (args[n], XmNactivateCallback, invers_cbliste);  n++;
    XtCreateManagedWidget ("invers", xmPushButtonWidgetClass,
                           box, args, n);
    
    /* XmPushButton-Widget als Exit-Button erzeugen
     */
    exitButton = XtCreateManagedWidget ("exit", xmPushButtonWidgetClass,
                                        box, (Arg*)NULL, 0);
    XtAddCallback (exitButton, XmNactivateCallback,
                   exitCB, (XtPointer)NULL);
}


/* inversCB
 *  - Callback: Vorder- und Hintergrundfarben des den Callback
 *              aufrufenden Widgets "widget" vertauschen
 */
/*ARGSUSED*/
void inversCB (widget, clientDaten, aufrufDaten)
Widget    widget;         /* den Callback aufrufendes Widget */
XtPointer clientDaten;    /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;    /* Aufruf-Daten (hier nicht verwendet) */
{
    Arg      args[10];       /* Argumente-Liste */ 
    Cardinal n;              /* Lauf-Index */
    Pixel    fg, bg;         /* Farbwerte */

    /* Vorder- und Hintergrundfarbwerte auslesen ...
     */
    n = 0;
    XtSetArg (args[n], XmNforeground, &fg);  n++;
    XtSetArg (args[n], XmNbackground, &bg);  n++;
    XtGetValues (widget, args, n);

    /* ... und umgekehrt wieder setzen
     */
    n = 0;
    XtSetArg (args[n], XmNforeground, bg);  n++;
    XtSetArg (args[n], XmNbackground, fg);  n++;
    XtSetValues (widget, args, n);
}


/* exitCB
 *  - Exit-Callback: Programm mit fehlerfreiem Exit-Status beenden
 */
/*ARGSUSED*/
void exitCB (widget, clientDaten, aufrufDaten)
Widget    widget;         /* den Callback aufrufendes Widget */
XtPointer clientDaten;    /* Client-Daten (hier nicht verwendet) */
XtPointer aufrufDaten;    /* Aufruf-Daten (hier nicht verwendet) */
{
    exit (EXIT_SUCCESS);
}

